@@ -31,6 +31,7 @@
 #else
 #ifdef sparc
 #include <alloca.h>
+extern "C" void *__builtin_alloca(...);
 #else
 #ifdef _AIX
 #pragma alloca
@@ -223,7 +224,7 @@ enum regexpcode
 
 /* Store NUMBER in two contiguous bytes starting at DESTINATION.  */
 #define STORE_NUMBER(destination, number)				\
-  { (destination)[0] = (number) & 0377;					\
+  { (destination)[0] = (char)((number) & 0377);				\
     (destination)[1] = (number) >> 8; }
   
 /* Same as STORE_NUMBER, except increment the destination pointer to
@@ -2504,7 +2505,7 @@ re_match_2 (struct re_pattern_buffer *pbufp,
           if (d >= string1 && d <= end1)
 	    dend = end_match_1;
           /* Restore register info.  */
-          last_used_reg = (short) *--stackp;
+          last_used_reg = (short) (int) *--stackp;
           
           /* Make the ones that weren't saved -1 or 0 again.  */
           for (this_reg = RE_NREGS - 1; this_reg > last_used_reg; this_reg--)
@@ -2550,7 +2551,7 @@ bcmp_translate (char *s1, char *s2, int len, unsigned char *translate)
 
 /* Entry points compatible with 4.2 BSD regex library.  */
 
-#ifndef emacs
+#if 0
 
 static struct re_pattern_buffer re_comp_buf;
 
