@@ -64,7 +64,8 @@ int
 _IO_file_doallocate(fp)
      register _IO_FILE *fp;
 {
-  register size_t size, couldbetty;
+  register _IO_size_t size;
+  int couldbetty;
   register char *p;
   struct stat st;
 
