@@ -34,6 +34,8 @@ _IO_fread(buf, size, count, fp)
   _IO_size_t bytes_requested = size*count;
   _IO_size_t bytes_read;
   COERCE_FILE(fp);
+  if (bytes_requested == 0)
+    return 0;
   bytes_read = _IO_sgetn(fp, (char *)buf, bytes_requested);
   return bytes_requested == bytes_read ? count : bytes_read / size;
 }
