@@ -29,6 +29,14 @@ the executable file might be covered by the GNU General Public License. */
 #if _IO_HAVE_SYS_WAIT
 #include <signal.h>
 #include <unistd.h>
+#ifdef __STDC__
+/* This is now provided by gcc's fixproto. */
+#include <stdlib.h>
+#endif
+#include <errno.h>
+#ifndef errno
+extern int errno;
+#endif
 #include <sys/wait.h>
 
 #ifndef _IO_fork
@@ -48,6 +56,10 @@ extern int _IO_pipe();
 extern int _IO_dup2();
 #endif
 
+#ifndef _IO_waitpid
+#define _IO_waitpid waitpid
+#endif
+
 #ifndef _IO_execl
 #define _IO_execl execl
 #endif
@@ -55,18 +67,17 @@ extern int _IO_dup2();
 #define _IO__exit _exit
 #endif
 
-#ifdef TODO
-/* Ditto for signal, sigxxxmask, wait etc */
-#endif
-
 struct _IO_proc_file
 {
   struct _IO_FILE_plus _file;
   /* Following fields must match those in class procbuf (procbuf.h) */
   _IO_pid_t _pid;
+  struct _IO_proc_file *_next;
 };
 typedef struct _IO_proc_file _IO_proc_file;
 
+static struct _IO_proc_file *proc_file_chain = NULL;
+
 _IO_FILE *
 _IO_proc_open(fp, command, mode)
      _IO_FILE* fp;
@@ -104,6 +115,15 @@ _IO_proc_open(fp, command, mode)
 	  _IO_dup2(child_end, child_std_end);
 	  _IO_close(child_end);
 	}
+      /* Posix.2:  "popen() shall ensure that any streams from previous
+         popen() calls that remain open in the parent process are closed
+	 in the new child process." */
+      while (proc_file_chain)
+	{
+	  _IO_close (_IO_fileno ((_IO_FILE *) proc_file_chain));
+	  proc_file_chain = proc_file_chain->_next;
+	}
+
       _IO_execl("/bin/sh", "sh", "-c", command, NULL);
       _IO__exit(127);
     }
@@ -114,6 +134,11 @@ _IO_proc_open(fp, command, mode)
       return NULL;
     }
   _IO_fileno(fp) = parent_end;
+
+  /* Link into proc_file_chain. */
+  ((_IO_proc_file*)fp)->_next = proc_file_chain;
+  proc_file_chain = (_IO_proc_file*)fp;
+
   fp->_IO_file_flags
     = read_or_write | (fp->_IO_file_flags & ~(_IO_NO_READS|_IO_NO_WRITES));
   return fp;
@@ -129,11 +154,11 @@ _IO_popen(command, mode)
 {
   _IO_proc_file *fpx = (_IO_proc_file*)malloc(sizeof(_IO_proc_file));
   _IO_FILE *fp = (_IO_FILE*)fpx;
+  if (fp == NULL)
+    return NULL;
   _IO_init(fp, 0);
   fp->_jumps = &_IO_proc_jumps;
   _IO_file_init(fp);
-  if (fp == NULL)
-    return NULL;
   ((struct _IO_FILE_plus*)fp)->_vtable = NULL;
   if (_IO_proc_open (fp, command, mode) != NULL)
     return fp;
@@ -148,45 +173,33 @@ _IO_proc_close(fp)
   /* This is not name-space clean. FIXME! */
 #if _IO_HAVE_SYS_WAIT
   int wstatus;
+  _IO_proc_file **ptr = &proc_file_chain;
   _IO_pid_t wait_pid;
   int status = _IO_close(_IO_fileno(fp));
-#if defined(SIG_BLOCK) && defined(SIG_SETMASK)
-  sigset_t set, oset;
-#endif
+
+  /* Unlink from proc_file_chain. */
+  for ( ; *ptr != NULL; ptr = &(*ptr)->_next)
+    {
+      if (*ptr == (_IO_proc_file*)fp)
+	{
+	  *ptr = (*ptr)->_next;
+	  break;
+	}
+    }
+
   if (status < 0)
     return status;
-#if defined(SIG_BLOCK) && defined(SIG_SETMASK)
-  sigemptyset (&set);
-  sigaddset (&set, SIGINT);
-  sigaddset (&set, SIGQUIT);
-  sigaddset (&set, SIGHUP);
-  sigprocmask (SIG_BLOCK, &set, &oset);
-#else
-#ifdef USE_SIGMASK
-  int mask = sigblock(sigmask(SIGINT) | sigmask(SIGQUIT) | sigmask(SIGHUP));
-#else
-  typedef void (*void_func)(int);
-  void_func intsave = (void_func)signal(SIGINT, SIG_IGN);
-  void_func quitsave = (void_func)signal(SIGQUIT, SIG_IGN);
-  void_func hupsave = (void_func)signal(SIGHUP, SIG_IGN);
-#endif
-#endif
-  while ((wait_pid = wait(&wstatus)) != ((_IO_proc_file*)fp)->_pid
-	 && wait_pid != -1) { }
-#if defined(SIG_BLOCK) && defined(SIG_SETMASK)
-  sigprocmask (SIG_SETMASK, &oset, (sigset_t *)NULL);
-#else
-#ifdef USE_SIGMASK
-  (void) sigsetmask(mask);
-#else
-  signal(SIGINT, intsave);
-  signal(SIGQUIT, quitsave);
-  signal(SIGHUP, hupsave);
-#endif
-#endif
+  /* POSIX.2 Rationale:  "Some historical implementations either block
+     or ignore the signals SIGINT, SIGQUIT, and SIGHUP while waiting
+     for the child process to terminate.  Since this behavior is not
+     described in POSIX.2, such implementations are not conforming." */
+  do
+    {
+      wait_pid = _IO_waitpid (((_IO_proc_file*)fp)->_pid, &wstatus, 0);
+    } while (wait_pid == -1 && errno == EINTR);
   if (wait_pid == -1)
     return -1;
-  return 0;
+  return wstatus;
 #else /* !_IO_HAVE_SYS_WAIT */
   return -1;
 #endif
