@@ -29,6 +29,6 @@ inline _IO_seekflags
 convert_to_seekflags(int dir, int mode)
 {
   return (_IO_seekflags)((int)dir
-			 | (mode & ios::in ? 0 : _IO_seek_not_in)
-			 | (mode & ios::out ? 0 : _IO_seek_not_out));
+			 | (mode & ios::in ? _IO_seek_set : _IO_seek_not_in)
+			 | (mode & ios::out ? _IO_seek_set : _IO_seek_not_out));
 }
