@@ -149,8 +149,10 @@ _IO_vfscanf(fp, fmt0, ap, errp)
 		if (isspace(c)) {
 			for (;;) {
 			        c = _IO_getc(fp);
-				if (c == EOF)
-				    goto eof_failure;
+				if (c == EOF) {
+				  seen_eof++;
+				  break;
+				}
 				if (!isspace(c)) {
 				    _IO_ungetc (c, fp);
 				    break;
