@@ -2,7 +2,7 @@
 #include "stdio.h"
 
 int
-fputc(c. fp)
+fputc(c, fp)
      int c;
      FILE *fp;
 {
